import * as PropTypes from 'prop-types';
import * as React from 'react';
import * as Point from './Point';
import * as Size from './Size';
const truncRound = (num) => Math.trunc(num * 10000) / 10000;
var GUESTURE_TYPE;
(function (GUESTURE_TYPE) {
    GUESTURE_TYPE["UNSET"] = "GUESTURE_TYPE_UNSET";
    GUESTURE_TYPE["PAN"] = "GUESTURE_TYPE_PAN";
    GUESTURE_TYPE["PINCH"] = "GUESTURE_TYPE_PINCH";
})(GUESTURE_TYPE || (GUESTURE_TYPE = {}));
class PinchToZoom extends React.Component {
    static getTouchesCoordinate(syntheticEvent) {
        const { currentTarget: { parentNode }, nativeEvent, } = syntheticEvent;
        if (!(parentNode instanceof HTMLElement) ||
            !(nativeEvent instanceof TouchEvent)) {
            return [];
        }
        const containerRect = parentNode.getBoundingClientRect();
        const rect = {
            origin: { x: containerRect.left, y: containerRect.top },
            size: {
                width: containerRect.width,
                height: containerRect.height,
            },
        };
        const { touches: touchList } = nativeEvent;
        const coordinates = [];
        for (let i = 0; i < touchList.length; i += 1) {
            const touch = touchList.item(i);
            if (touch) {
                const touchPoint = {
                    x: touch.clientX,
                    y: touch.clientY,
                };
                const p = Point.normalizePointInRect(touchPoint, rect);
                coordinates.push(p);
            }
        }
        return coordinates;
    }
    constructor(props) {
        super(props);
        this.transform = {
            zoomFactor: 1.0,
            translate: Point.newOriginPoint(),
        };
        this.currentGesture = GUESTURE_TYPE.UNSET;
        this.pinchStartZoomFactor = 1.0;
        this.pinchStartTouchMidpoint = Point.newOriginPoint();
        this.pinchStartTranslate = Point.newOriginPoint();
        this.pinchStartTouchPointDist = 0;
        this.panStartPoint = Point.newOriginPoint();
        this.panStartTranslate = Point.newOriginPoint();
        this.state = {
            lastSingleTouchPoint: Point.newOriginPoint(),
        };
    }
    componentDidUpdate(prevProps) {
        if (prevProps.minZoomScale !== this.props.minZoomScale ||
            prevProps.boundSize.height !== this.props.boundSize.height) {
            this.zoomContentArea(this.props.minZoomScale);
            this.guardZoomAreaTranslate();
        }
    }
    onPinchStart(syntheticEvent) {
        const [p1, p2] = PinchToZoom.getTouchesCoordinate(syntheticEvent);
        this.pinchStartTouchMidpoint = Point.midpoint(p1, p2);
        this.pinchStartTouchPointDist = Point.distance(p1, p2);
        const { currentZoomFactor, currentTranslate } = this.getTransform();
        this.pinchStartZoomFactor = currentZoomFactor;
        this.pinchStartTranslate = currentTranslate;
    }
    onPinchMove(syntheticEvent) {
        const [p1, p2] = PinchToZoom.getTouchesCoordinate(syntheticEvent);
        const pinchCurrentTouchPointDist = Point.distance(p1, p2);
        const deltaTouchPointDist = pinchCurrentTouchPointDist - this.pinchStartTouchPointDist;
        const newZoomFactor = this.pinchStartZoomFactor + deltaTouchPointDist * 0.01;
        this.zoomContentArea(newZoomFactor);
    }
    onPinchEnd() {
        this.guardZoomAreaScale();
        this.guardZoomAreaTranslate();
    }
    onPanStart(syntheticEvent) {
        const [p1] = PinchToZoom.getTouchesCoordinate(syntheticEvent);
        const { currentTranslate } = this.getTransform();
        this.panStartPoint = p1;
        this.panStartTranslate = currentTranslate;
    }
    onPanMove(syntheticEvent) {
        const [dragPoint] = PinchToZoom.getTouchesCoordinate(syntheticEvent);
        const { currentZoomFactor } = this.getTransform();
        const origin = this.panStartPoint;
        const prevTranslate = this.panStartTranslate;
        const dragOffset = Point.offset(dragPoint, origin);
        const adjustedZoomOffset = Point.scale(dragOffset, 1 / currentZoomFactor);
        const nextTranslate = Point.sum(adjustedZoomOffset, prevTranslate);
        this.panContentArea(nextTranslate);
    }
    onPanEnd() {
        this.guardZoomAreaTranslate();
    }
    guardZoomAreaScale() {
        const { currentZoomFactor } = this.getTransform();
        const { minZoomScale, maxZoomScale } = this.props;
        if (currentZoomFactor > maxZoomScale) {
            this.zoomContentArea(maxZoomScale);
        }
        else if (currentZoomFactor < minZoomScale) {
            this.zoomContentArea(minZoomScale);
        }
    }
    guardZoomAreaTranslate() {
        if (!this.zoomAreaContainer || !this.zoomArea) {
            return;
        }
        const { currentZoomFactor, currentTranslate } = this.getTransform();
        const { minZoomScale } = this.props;
        const { clientWidth: containerW, clientHeight: containerH, } = this.zoomAreaContainer;
        const { clientWidth: contentW, clientHeight: contentH } = this.zoomArea;
        if (currentZoomFactor < minZoomScale) {
            return;
        }
        const boundSize = {
            width: containerW,
            height: containerH,
        };
        const contentSize = Size.scale({
            width: contentW,
            height: contentH,
        }, currentZoomFactor);
        const diff = Size.diff(boundSize, contentSize);
        const diffInPoint = Size.toPoint(diff);
        const unitScaleLeftTopPoint = Point.scale(diffInPoint, 1 / (2 * currentZoomFactor));
        const maxLeftTopPoint = Point.boundWithin(Point.newOriginPoint(), unitScaleLeftTopPoint, Point.map(unitScaleLeftTopPoint, truncRound));
        const unitScaleRightBottomPoint = Point.scale(diffInPoint, 1 / currentZoomFactor);
        const maxRightBottomPoint = {
            x: Math.min(unitScaleRightBottomPoint.x, maxLeftTopPoint.x),
            y: Math.min(unitScaleRightBottomPoint.y, maxLeftTopPoint.y),
        };
        const validatePos = Point.boundWithin(maxRightBottomPoint, currentTranslate, maxLeftTopPoint);
        if (!Point.isEqual(validatePos, currentTranslate)) {
            this.panContentArea(validatePos);
        }
    }
    panContentArea(pos) {
        this.setTransform({
            translate: pos,
        });
    }
    zoomContentArea(zoomFactor) {
        if (!this.zoomAreaContainer || !this.zoomArea) {
            return;
        }
        const prevZoomFactor = this.pinchStartZoomFactor;
        const prevTranslate = this.pinchStartTranslate;
        const { clientWidth: containerW, clientHeight: containerH, } = this.zoomAreaContainer;
        const boundSize = {
            width: containerW,
            height: containerH,
        };
        const prevZoomSize = Size.scale(boundSize, prevZoomFactor);
        const nextZoomSize = Size.scale(boundSize, zoomFactor);
        const prevRectCenterPoint = {
            x: prevZoomSize.width / 2,
            y: prevZoomSize.height / 2,
        };
        const nextRectCenterPoint = {
            x: nextZoomSize.width / 2,
            y: nextZoomSize.height / 2,
        };
        const deltaTranslate = Point.scale(Point.offset(prevRectCenterPoint, nextRectCenterPoint), 1 / (zoomFactor * prevZoomFactor));
        const accumulateTranslate = Point.sum(deltaTranslate, prevTranslate);
        this.setTransform({
            zoomFactor: truncRound(zoomFactor),
            translate: accumulateTranslate,
        });
    }
    handleTouchStart(syntheticEvent) {
        if (!this.zoomAreaContainer || !this.zoomArea) {
            return;
        }
        const { nativeEvent } = syntheticEvent;
        if (!(nativeEvent instanceof TouchEvent)) {
            return;
        }
        this.zoomArea.style.transitionDuration = '0.0s';
        switch (nativeEvent.touches.length) {
            case 2:
                this.currentGesture = GUESTURE_TYPE.PINCH;
                this.onPinchStart(syntheticEvent);
                break;
            default: {
                const [p1] = PinchToZoom.getTouchesCoordinate(syntheticEvent);
                this.setState({ lastSingleTouchPoint: p1 });
                this.currentGesture = GUESTURE_TYPE.PAN;
                this.onPanStart(syntheticEvent);
            }
        }
    }
    handleTouchMove(syntheticEvent) {
        const { nativeEvent } = syntheticEvent;
        if (!(nativeEvent instanceof TouchEvent)) {
            return;
        }
        switch (nativeEvent.touches.length) {
            case 2:
                if (this.currentGesture === GUESTURE_TYPE.PINCH) {
                    this.onPinchMove(syntheticEvent);
                }
                break;
            default:
                if (this.currentGesture === GUESTURE_TYPE.PAN) {
                    this.onPanMove(syntheticEvent);
                }
        }
    }
    handleTouchEnd(syntheticEvent) {
        if (!this.zoomAreaContainer || !this.zoomArea) {
            return;
        }
        this.zoomArea.style.transitionDuration = '0.3s';
        if (this.currentGesture === GUESTURE_TYPE.PINCH) {
            this.onPinchEnd();
        }
        if (this.currentGesture === GUESTURE_TYPE.PAN) {
            this.onPanEnd();
        }
        this.currentGesture = GUESTURE_TYPE.UNSET;
    }
    autoZoomToLastTouchPoint() {
        const { lastSingleTouchPoint } = this.state;
        if (lastSingleTouchPoint.x === 0 && lastSingleTouchPoint.y === 0) {
            return;
        }
        this.autoZoomToPosition(lastSingleTouchPoint);
    }
    autoZoomToPosition(pos) {
        if (!this.zoomAreaContainer || !this.zoomArea) {
            return;
        }
        const autoZoomFactor = 2.0;
        const { currentZoomFactor, currentTranslate } = this.getTransform();
        const zoomAreaContainerW = this.zoomAreaContainer.clientWidth;
        const zoomAreaContainerH = this.zoomAreaContainer.clientHeight;
        const zoomAreaX = (pos.x / currentZoomFactor - currentTranslate.x) * autoZoomFactor;
        const zoomAreaY = (pos.y / currentZoomFactor - currentTranslate.y) * autoZoomFactor;
        const deltaX = zoomAreaContainerW / 2 - zoomAreaX;
        const deltaY = zoomAreaContainerH / 2 - zoomAreaY;
        const inScaleTranslate = {
            x: deltaX / autoZoomFactor,
            y: deltaY / autoZoomFactor,
        };
        this.zoomArea.style.transitionDuration = '0.3s';
        this.setTransform({
            zoomFactor: autoZoomFactor,
            translate: {
                x: inScaleTranslate.x,
                y: inScaleTranslate.y,
            },
        });
        this.guardZoomAreaTranslate();
    }
    setTransform({ zoomFactor = this.transform.zoomFactor, translate = {
        x: this.transform.translate.x,
        y: this.transform.translate.y,
    }, } = {}) {
        if (!this.zoomAreaContainer || !this.zoomArea) {
            return;
        }
        const roundTransalteX = Math.round(translate.x * 1000) / 1000;
        const roundTransalteY = Math.round(translate.y * 1000) / 1000;
        if (zoomFactor < this.props.minZoomScale * 0.8) {
            return;
        }
        this.transform.zoomFactor = zoomFactor;
        this.transform.translate.x = roundTransalteX;
        this.transform.translate.y = roundTransalteY;
        const styleString = `
        scale(${zoomFactor})
        translate(${roundTransalteX}px, ${roundTransalteY}px)
        translateZ(${0})
      `;
        this.zoomArea.style.transform = styleString;
        this.zoomArea.style.webkitTransform = styleString;
    }
    getTransform() {
        const { zoomFactor, translate } = this.transform;
        return {
            currentZoomFactor: zoomFactor,
            currentTranslate: {
                x: translate.x,
                y: translate.y,
            },
        };
    }
    render() {
        const { debug, className, children } = this.props;
        const classNameList = ['', 'pinch-to-zoom-container'];
        const containerInlineStyle = {
            display: 'inline-block',
            overflow: 'hidden',
            backgroundColor: 'inherit',
        };
        const zoomAreaInlineStyle = {
            display: 'inline-block',
            willChange: 'transform',
            transformOrigin: '0px 0px 0px',
            transition: 'transform 0ms ease',
            transitionTimingFunction: 'cubic-bezier(0.1, 0.57, 0.1, 1)',
            transitionDuration: '0ms',
            perspective: 1000,
            width: '100%',
        };
        if (debug) {
            classNameList.push('debug');
            containerInlineStyle.backgroundColor = 'red';
        }
        return (React.createElement("div", { className: className.concat(classNameList.join(' ')), style: containerInlineStyle, onTouchStart: e => this.handleTouchStart(e), onTouchMove: e => this.handleTouchMove(e), onTouchEnd: e => this.handleTouchEnd(e), ref: c => {
                this.zoomAreaContainer = c || undefined;
            } },
            React.createElement("div", { className: "pinch-to-zoom-area", style: zoomAreaInlineStyle, ref: c => {
                    this.zoomArea = c || undefined;
                } }, children)));
    }
}
PinchToZoom.defaultProps = {
    debug: false,
    className: '',
    minZoomScale: 1.0,
    maxZoomScale: 4.0,
    boundSize: {
        width: 100,
        height: 100,
    },
    contentSize: {
        width: 100,
        height: 100,
    },
};
PinchToZoom.propTypes = {
    debug: PropTypes.bool,
    className: PropTypes.string,
    minZoomScale: PropTypes.number,
    maxZoomScale: PropTypes.number,
    boundSize: PropTypes.shape({
        width: PropTypes.number,
        height: PropTypes.number,
    }),
    contentSize: PropTypes.shape({
        width: PropTypes.number,
        height: PropTypes.number,
    }),
    children: PropTypes.node,
};
export default PinchToZoom;
//# sourceMappingURL=index.js.map